==> RangeMinQuery-0.1.1.tar.gz: RangeMinQuery-0.1.1/RangeMinQuery/SegmentTree.py <==
#! /usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import *

class SegmentTreeNode(object):
    def __init__(self, nodeRange, default=None):
        self.nodeRange = nodeRange
        self.value = default
        self.children = []
        self.parent = None

class SegmentTree(object):
    @staticmethod
    def __covers(a, b):
        return a[0] <= b[0] and b[1] <= a[1]

    @staticmethod
    def __intersects(a, b):
        return not (a[1] < b[0] or a[0] > b[1])

    def __init__(self, keys, func, default=None, maxChildNum=2):
        self.func = func

        l = [SegmentTreeNode((k, k), default) for k in sorted(keys)]
        self.mapping = {n.nodeRange[0]: n for n in l}

        while len(l) > 1:
            nl = []

            for i in range(0, len(l), maxChildNum):
                c = l[i:i+maxChildNum]
                n = SegmentTreeNode((c[0].nodeRange[0], c[-1].nodeRange[1]), default)
                n.children = c
                for x in c:
                    x.parent = n

                nl.append(n)

            l = nl

        self.root = l[0]

    def update(self, keyValueDict):
        ul = set()

        for (k, v) in keyValueDict.items():
            node = self.mapping[k]

            if node.value != v:
                node.value = v
                if node.parent != None:
                    ul.add(node.parent)

        while len(ul) > 0:
            newUL = set()

            for node in ul:
                pv = self.func(x.value for x in node.children)
                if pv != node.value:
                    node.value = pv
                    if node.parent != None:
                        newUL.add(node.parent)

            ul = newUL

    def query(self, queryRange):
        def query_aux(node):
            if SegmentTree.__covers(queryRange, node.nodeRange):
                return node.value

            return self.func(
                query_aux(cNode)
                for cNode in node.children
                if SegmentTree.__intersects(queryRange, cNode.nodeRange)
            )

        if queryRange[0] > queryRange[1]:
            raise

        if not SegmentTree.__intersects(queryRange, self.root.nodeRange):
            raise

        return query_aux(self.root)

==> RangeMinQuery-0.1.2.tar.gz: RangeMinQuery-0.1.2/RangeMinQuery/SegmentTree.py <==
#! /usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import *

class SegmentTreeNode(object):
    def __init__(self, nodeRange, default=None):
        self.nodeRange = nodeRange
        self.value = default
        self.children = []
        self.parent = None


class SegmentTree(object):
    @staticmethod
    def __covers(a, b):
        return a[0] <= b[0] and b[1] <= a[1]


    @staticmethod
    def __intersects(a, b):
        return not (a[1] < b[0] or a[0] > b[1])


    def __init__(self, keys, func, default=None, maxChildNum=2):
        self.func = func

        l = [
            SegmentTreeNode((k, k), default)
            for k in sorted(keys)
        ]
        self.mapping = {n.nodeRange[0]: n for n in l}

        while len(l) > 1:
            nl = []

            for i in range(0, len(l), maxChildNum):
                c = l[i:i + maxChildNum]
                n = SegmentTreeNode(
                    (c[0].nodeRange[0], c[-1].nodeRange[1]),
                    default
                )
                n.children = c
                for x in c:
                    x.parent = n

                nl.append(n)

            l = nl

        self.root = l[0]


    def update(self, keyValueDict):
        ul = set()

        for (k, v) in keyValueDict.items():
            node = self.mapping[k]

            if node.value != v:
                node.value = v
                if node.parent != None:
                    ul.add(node.parent)

        while len(ul) > 0:
            newUL = set()

            for node in ul:
                pv = self.func(x.value for x in node.children)
                if pv != node.value:
                    node.value = pv
                    if node.parent != None:
                        newUL.add(node.parent)

            ul = newUL


    def query(self, queryRange):
        def query_aux(node):
            if SegmentTree.__covers(queryRange, node.nodeRange):
                return node.value

            return self.func(
                query_aux(cNode)
                for cNode in node.children
                if SegmentTree.__intersects(queryRange, cNode.nodeRange)
            )


        if queryRange[0] > queryRange[1]:
            raise

        if not SegmentTree.__intersects(queryRange, self.root.nodeRange):
            raise

        return query_aux(self.root)
